MODDIR=${0%/*}
soc_model=$(getprop ro.soc.model)

lock_val() {
    for file in $(find $2); do
        file="$(realpath $file)"
        umount "$file"
        chmod +w "$file"
        echo "$1" >"$file"
        chmod -w "$file"
        restorecon -R -F "$file" > /dev/null 2>&1
    done
}

lock_val_in_path() {
    for file in $(find $2 -name $3); do
        lock_val "$1" "$file"
    done
}

mask_val() {
    for file in $(find $2); do
        lock_val "$1" "$file"

        TIME="$(date "+%s%N")"
        echo "$1" >"/dev/mount_masks/mount_mask_$TIME"
        mount --bind "/dev/mount_masks/mount_mask_$TIME" "$file"
        restorecon -R -F "$file" > /dev/null 2>&1
    done
}

mask_val_in_path() {
    for file in $(find $2 -name $3); do
        mask_val "$1" "$file"
    done
}

lock_perfhal_min() {
    mask_val "0:$1 1:$1 2:$1 3:$1 4:$1 5:$1 6:$1 7:$1" "/sys/kernel/msm_performance/parameters/cpu_min_freq"
}

lock_perfhal_max() {
    mask_val "0:$1 1:$1 2:$1 3:$1 4:$1 5:$1 6:$1 7:$1" "/sys/kernel/msm_performance/parameters/cpu_max_freq"
}

lock_perfhal() {
    lock_perfhal_min "$1"
    lock_perfhal_max "$2"
}

wait_until_boot_complete() {
    while [ "$(getprop sys.boot_completed)" != "1" ]; do
        sleep 1
    done
}

wait_until_login() {
    until [ -d "/data/data/android" ]; do
        sleep 1
    done
}

set_governor() {
    lock_val "$1" "/sys/devices/system/cpu/cpu*/cpufreq/scaling_governor"
}

set_corectl() {
    mask_val "$1" "/sys/devices/system/cpu/cpu*/core_ctl/enable"
}

write() {
    chmod +w $2
    echo "$1" | tee $2 > /dev/null 2>&1
}

# GKI modification by/from Hamjin
init(){
    echo "$(pidof lspd)" >/dev/cpuset/background/cgroup.procs
    echo "$(pidof logd)" >/dev/cpuset/background/cgroup.procs
    echo "$(pidof traced_probes)" >/dev/cpuset/background/cgroup.procs
    
    lock_val "0" /sys/kernel/rcu_expedited
    mask_val "2" /proc/sys/kernel/sched_pelt_multiplier
    
    lock_val "disabled" "/sys/class/thermal/thermal_zone*/mode"
    
    #lock_val "92" "/sys/class/kgsl/kgsl-3d0/devfreq/mod_percent"

    mount -t debugfs none /sys/kernel/debug
    write "5000000" "/sys/kernel/debug/sched/migration_cost_ns"
    write "12500000" "/sys/kernel/debug/sched/wakeup_granularity_ns"
}

init_zram() {
    if ! grep -q zram /proc/swaps; then
        mem_size=$(cat /proc/meminfo | awk 'NR==1{print $2}')
        for i in $(seq 0 1); do
            lock_val "0" /sys/class/block/zram$i/mem_limit
            echo "$mem_size" > /sys/block/zram$i/hybridswap_zram_increase
        done
        lock_val "2*$mem_size" /dev/memcg/memory.zram_used_limit_mb
        echo "200" > /sys/module/oplus_bsp_zram_opt/parameters/direct_vm_swappiness
        echo "2048" > /sys/module/oplus_bsp_zram_opt/parameters/swappiness_threshold1_size
        echo "1024" > /sys/module/oplus_bsp_zram_opt/parameters/swappiness_threshold2_size
    fi
}

init_mem() {
    sleep 180s

    mask_val "200" /proc/sys/vm/swappiness
    mask_val "1" /proc/sys/vm/page-cluster
    mask_val "65536" /proc/sys/vm/min_free_kbytes
    mask_val "262144" /proc/sys/vm/extra_free_kbytes
    mask_val "100" /proc/sys/vm/watermark_scale_factor
    mask_val "15000" /proc/sys/vm/watermark_boost_factor
    mask_val "1" /proc/sys/vm/overcommit_memory

    lock_val "Y" /sys/kernel/mm/lru_gen/enabled
    lock_val "1000" /sys/kernel/mm/lru_gen/min_ttl_ms
}

init_android_config() {
    device_config set_sync_disabled_for_tests until_reboot
    device_config put activity_manager max_cached_processes 65535
    device_config put activity_manager max_phantom_processes 65535
    device_config put netd_native happy_eyeballs_enable true
    device_config put netd_native parallel_lookup true
    device_config put netd_native sort_nameservers true
    device_config put runtime_native_boot disable_lock_profiling true
    device_config put lmkd_native use_minfree_levels false
    device_config delete lmkd_native thrashing_limit_critical
    device_config put activity_manager use_compaction false

    settings put global settings_enable_monitor_phantom_procs false
}
# GKI modification fin.

init_oplus_common_config(){
    if [ "$(getprop sys.pxlw.iris.support)" ]; then
        echo "1" > /sys/kernel/iris/lp_opt
        echo "1" > /sys/kernel/iris/abyp_opt
    fi
    
    mask_val "1000" /proc/oplus-votable/GAUGE_UPDATE/force_val
    mask_val "1" /proc/oplus-votable/GAUGE_UPDATE/force_active

    case $soc_model in
    SM8450|SM8475|SM7475)
        lock_val "0-6" /dev/cpuset/display/cpus
        lock_val "0-6" /dev/cpuset/audio-app/cpus
        lock_val "0-7" /dev/cpuset/system-background/cpus
        lock_val "0-3" /dev/cpuset/h-background/cpus
        lock_val "0-3" /dev/cpuset/l-background/cpus
        lock_val "0-7" /dev/cpuset/foreground/cpus
        lock_val "0-7" /dev/cpuset/top-app/cpus
        lock_val "0-3" /dev/cpuset/background/cpus
        lock_val "0-6" /dev/cpuset/kswapd-like/cpus
        for i in 0 4 7; do
            echo "1" > "/sys/devices/system/cpu/cpufreq/policy$i/uag/stall_aware"
            echo "85" > "/sys/devices/system/cpu/cpufreq/policy$i/uag/reduce_pct_of_stall"
        done
        ;;
    SM8550)
        lock_val "3-6" /dev/cpuset/display/cpus
        lock_val "3-6" /dev/cpuset/sf/cpus
        lock_val "0-6" /dev/cpuset/audio-app/cpus
        lock_val "0-7" /dev/cpuset/system-background/cpus
        #lock_val "0-2" /dev/cpuset/h-background/cpus
        #lock_val "0-2" /dev/cpuset/l-background/cpus
        lock_val "0-6" /dev/cpuset/foreground/cpus
        lock_val "3-7" /dev/cpuset/top-app/cpus
        lock_val "0-2" /dev/cpuset/background/cpus
        lock_val "0-6" /dev/cpuset/kswapd-like/cpus
        mask_val "1" /sys/devices/system/cpu/bus_dcvs/DDRQOS/boost_freq
        lock_val "307200,45000,1228800,52000,0" "/sys/module/cpufreq_effiency/parameters/cluster0_effiency"
        lock_val "499200,50000,1785600,55000,0" "/sys/module/cpufreq_effiency/parameters/cluster1_effiency"
        lock_val "595200,55000,2092800,60000,0" "/sys/module/cpufreq_effiency/parameters/cluster2_effiency"
        for i in 0 3 7; do
            echo "1" > "/sys/devices/system/cpu/cpufreq/policy$i/uag/stall_aware"
            echo "90" > "/sys/devices/system/cpu/cpufreq/policy$i/uag/reduce_pct_of_stall"
        done
        echo "10" > "/sys/devices/system/cpu/cpufreq/policy0/uag/break_freq_margin"
        ;;
    SM7675|SM8635)
        lock_val "3-6" /dev/cpuset/display/cpus
        lock_val "3-6" /dev/cpuset/sf/cpus
        lock_val "0-6" /dev/cpuset/audio-app/cpus
        lock_val "0-7" /dev/cpuset/system-background/cpus
        lock_val "0-2" /dev/cpuset/h-background/cpus
        lock_val "0-2" /dev/cpuset/l-background/cpus
        lock_val "0-7" /dev/cpuset/foreground/cpus
        lock_val "3-7" /dev/cpuset/top-app/cpus
        lock_val "0-2" /dev/cpuset/background/cpus
        lock_val "0-6" /dev/cpuset/kswapd-like/cpus
        for i in 0 3 7; do
            echo "1" > "/sys/devices/system/cpu/cpufreq/policy$i/uag/stall_aware"
            echo "90" > "/sys/devices/system/cpu/cpufreq/policy$i/uag/reduce_pct_of_stall"
        done
        ;;
    SM8650)
        lock_val "2-6" /dev/cpuset/display/cpus
        lock_val "2-6" /dev/cpuset/sf/cpus
        lock_val "0-6" /dev/cpuset/audio-app/cpus
        lock_val "0-1" /dev/cpuset/background/cpus
        lock_val "0-7" /dev/cpuset/system-background/cpus
        lock_val "0-1" /dev/cpuset/h-background/cpus
        lock_val "0-1" /dev/cpuset/l-background/cpus
        lock_val "0-7" /dev/cpuset/foreground/cpus
        lock_val "2-7" /dev/cpuset/top-app/cpus
        lock_val "0-6" /dev/cpuset/kswapd-like/cpus
        lock_val "0-6" /dev/cpuset/camera-background/cpus
        lock_val "0-6" /dev/cpuset/camera-daemon/cpus
        mask_val "1" /sys/devices/system/cpu/bus_dcvs/DDRQOS/boost_freq
        for i in 0 2 5 7; do
            echo "1" > "/sys/devices/system/cpu/cpufreq/policy$i/uag/stall_aware"
            echo "90" > "/sys/devices/system/cpu/cpufreq/policy$i/uag/reduce_pct_of_stall"
        done
        echo "10" > "/sys/devices/system/cpu/cpufreq/policy0/uag/break_freq_margin"
        ;;
    SM8750)
        lock_val "0-5" /dev/cpuset/display/cpus
        lock_val "0-7" /dev/cpuset/sf/cpus
        lock_val "0-5" /dev/cpuset/audio-app/cpus
        lock_val "0-5" /dev/cpuset/background/cpus
        lock_val "0-7" /dev/cpuset/system-background/cpus
        lock_val "0-5" /dev/cpuset/h-background/cpus
        lock_val "0-5" /dev/cpuset/l-background/cpus
        lock_val "0-7" /dev/cpuset/foreground/cpus
        lock_val "0-7" /dev/cpuset/top-app/cpus
        lock_val "0-5" /dev/cpuset/kswapd-like/cpus
        lock_val "0-5" /dev/cpuset/camera-background/cpus
        lock_val "0-5" /dev/cpuset/camera-daemon/cpus
        mask_val "1" /sys/devices/system/cpu/bus_dcvs/DDRQOS/boost_freq
        lock_val "0" "/sys/devices/system/cpu/dmof/disable_memcpy_optimization"
        ;;
    *)
        lock_val "0-7" /dev/cpuset/system-background/cpus
        ;;
    esac
    
    lock_perfhal "0" "2147483647"
    
    stop perf-hal-2-3
    start perf-hal-2-3
    
    #stop vendor.oplus.ormsHalService-aidl-default
}

init_pandora(){
    BUS_DIR="/sys/devices/system/cpu/bus_dcvs"
    lock_val_in_path "2147483647" "$BUS_DIR" "max_freq"
    lock_val_in_path "0" "$BUS_DIR" "min_freq"
    write "libunity.so,libfb.so,libflutter.so,libapp.so" "/proc/sys/walt/sched_lib_name"
}

haptic_enhance(){
    echo "1800" > /sys/class/qcom-haptics/vmax
    echo "8300" > /sys/class/qcom-haptics/cl_vmax
    echo "11451" > /sys/class/qcom-haptics/fifo_vmax
}

ysvk(){
    sleep 20
    for yspak in Yuanshen ys.mi ys.bilibili GenshinImpact; do
        local ysdir="/data/media/0/Android/data/com.miHoYo.$yspak/files"
        if [ -d "$ysdir" ]; then
            cp "$MODDIR"/assets/hardware_model_config.json "$ysdir"
        fi
    done
}

init_gowass(){
    gowass=$(getprop fas-rs-installed)
    [ "$gowass" = "true" ] && echo "1" > /proc/game_opt/disable_cpufreq_limit
}

init_proc_cpuset(){
    echo "$(pgrep uaudio_svc)" > /dev/cpuset/audio-app/tasks
    echo "$(pgrep audioadsprpcd)" > /dev/cpuset/audio-app/tasks
    echo "$(pidof media.audio.qc.codec)" > /dev/cpuset/audio-app/cgroup.procs
    echo "$(pidof audioserver)" > /dev/cpuset/audio-app/cgroup.procs
    echo "$(pidof android.hardware.audio.service_64)" > /dev/cpuset/audio-app/cgroup.procs
    echo "$(pidof media.hwcodec)" > /dev/cpuset/audio-app/cgroup.procs
    echo "$(pidof media.swcodec)" > /dev/cpuset/audio-app/cgroup.procs
    echo "$(pidof update_engine)" > /dev/cpuset/top-app/cgroup.procs
    echo "$(pidof vendor-oplus-hardware-touch-V2-service)" > /dev/cpuset/sf/cgroup.procs
    echo "$(pidof touchDaemon)" > /dev/cpuset/sf/cgroup.procs
}

touch_fix(){
    renice -n -19 -p $(pidof vendor-oplus-hardware-touch-V2-service)
    renice -n -19 -p $(pidof touchDaemon)
    renice -n -20 -p $(pidof surfaceflinger)
    
    #It's not a good solution, thus possibility of abnormal interuption will happen unwillingly
}

init_afterboot(){
    sleep 60
    
    for i in $(seq 0 7); do
        echo 0 > "/sys/devices/system/cpu/cpufreq/policy$i/scaling_min_freq"
    done
}

mkdir -p /dev/mount_mask

# ZRAM opt
init_zram
# ZRAM opt fin.


wait_until_boot_complete

# Performance adjustment
if [ "$soc_model" != "SM8750" ]; then 
    init_pandora
fi

if [ ! -f "/data/adb/modules/sun_optimizer/disable" ]; then
    if [ ! -d "/data/adb/modules/sun_optimizer" -a ! -d "/data/adb/modules_update/sun_optimizer" ]; then
        init
        init_oplus_common_config
        wait_until_login
        sleep 10
        init_proc_cpuset
    fi
fi

touch_fix
# Performance adjustment fin.

# Vibration enhancement
haptic_enhance
# Vibration enhancement fin.

# MEM opt
if [ ! -f "/data/adb/modules/sun_optimizer/disable" ]; then
    if [ ! -d "/data/adb/modules/sun_optimizer" -a ! -d "/data/adb/modules_update/sun_optimizer" ]; then
        init_mem
    fi
fi
# MEM opt fin.

wait_until_login
ysvk

init_gowass
init_afterboot
touch_fix