model=$(getprop ro.product.name)
marketname=$(getprop ro.product.marketname)
brand=$(getprop ro.product.brand)
kernel_version=$(uname -r)
HYPROP="$MODPATH"/module.prop
DESC_ZH="赐予新生！为 $marketname！"
DESC_EN="Bestow a rebirth upon $marketname\."
DESC_FR="Accordez une renaissance à $marketname\."
hydro_version=$(grep_prop version "$HYPROP")
hydro_versionCode=$(grep_prop versionCode "$HYPROP")
settings="$MODPATH"/settings.txt
settings_old=/data/adb/modules/ab_optimizer/settings.txt

hydro_print(){
    sleep 0.1
    ui_print "$1"
}

hydro_abort(){
    ui_print ""
    ui_print ""
    ui_print ""
    abort
}

print_line(){
    hydro_print "——————————"
}

hydro_language_select(){
    hydro_print "请选择您的语言/Choose your language/Choisissez votre langue"
    hydro_print "音量键 + : 简体中文"
    hydro_print "Volume - : English/Français"
    while [ "$key_click_en" = "" ]; do
        key_click_en=$(getevent -qlc 1 | awk '{ print $3 }' | grep 'KEY_')
        sleep 0.2
    done
    case $key_click_en in
    KEY_VOLUMEUP)
        hydro_print "--已选择语言: 简体中文"
        lang="zh"
        sed -i "s/^description=.*$/description=$DESC_ZH/" "$HYPROP"
        ;;
    *)
        hydro_language_choose_fr_en
        ;;
    esac
    sleep 0.5
    print_line
}

hydro_language_choose_fr_en(){
    hydro_print "Volume + : Français"
    hydro_print "Volume - : English"
    while [ "$key_click_fr" = "" ]; do
        key_click_fr=$(getevent -qlc 1 | awk '{ print $3 }' | grep 'KEY_')
        sleep 0.2
    done
    case $key_click_fr in
    KEY_VOLUMEUP)
        hydro_print "--Langue actuelle : Français"
        lang="fr"
        sed -i "s/^description=.*$/description=$DESC_FR/" "$HYPROP"
        sed -i "s/Hydrostellaire 沨莹/Hydrostellaire/g" "$HYPROP"
        ;;
    *)
        hydro_print "--Current language: English"
        lang="en"
        sed -i "s/^description=.*$/description=$DESC_EN/" "$HYPROP"
        sed -i "s/Hydrostellaire 沨莹/Hydrostellaire/g" "$HYPROP"
        ;;
    esac
}

hydro_kernel_version(){
    if [[ $kernel_version == *"Yuni"* ]]; then
        h_yuni_kernel=1
    elif [[ $kernel_version == *"AngelBeats"* ]]; then
        h_yuni_kernel=2
    elif [[ $kernel_version == *"Pandora"* ]]; then
        h_yuni_kernel=3
    else
        h_yuni_kernel=0
    fi
    if [ -f "/data/adb/modules/pandora_kernel/remove" -o -f "/data/adb/modules/yuni_kernel/remove" -o -f "/data/adb/modules/angelbeats_kernel/remove" ]; then
        h_yuni_kernel=0
    fi
    if [[ $h_yuni_kernel -gt 0 ]]; then
        is_pandora=1
    else
        is_pandora=0
    fi
}

hydro_model_detect(){
    case "$model" in
    liuqin)
        mi_model="23046RP50C" # 6 Pro
        ;;
    nabu)
        mi_model="21051182C" # 5
        ;;
    *)
        case $lang in
        zh)
            hydro_print "未受支持的设备！"
            hydro_print "请确保你没有使用任何伪装机型的模块"
            hydro_abort
            ;;
        en)
            hydro_print "Unsupported device!"
            hydro_print "'Make sure you are not using any modules that disguise your model"
            hydro_abort
            ;;
        fr)
            hydro_print "Périphérique non pris en charge!"
            hydro_print "'Assurez-vous que vous n'utilisez aucun module qui masque votre modèle"
            hydro_abort
            ;;
        esac
        ;;
    esac
    case $lang in
    zh)
        hydro_print "--设备信息--"
        hydro_print "产品名称：$model"
        hydro_print "市场名称：$marketname"
        hydro_print "内核版本：$kernel_version"
        hydro_print "模块版本：$hydro_version ($hydro_versionCode)"
        ;;
    en)
        hydro_print "--Device info--"
        hydro_print "Product name: $model"
        hydro_print "Market name: $marketname"
        hydro_print "Kernel version: $kernel_version"
        hydro_print "Hydro version: $hydro_version ($hydro_versionCode)"
        ;;
    fr)
        hydro_print "--Info appareil--"
        hydro_print "Nom du produit: $model"
        hydro_print "Nom du marché: $marketname"
        hydro_print "Version du noyau: $kernel_version"
        hydro_print "Version du module: $hydro_version ($hydro_versionCode)"
        ;;
    esac
    if [[ "$KSU" == "true" ]]; then
        hydro_print "KernelSU: $KSU_VER_CODE"
        case $lang in
        zh)
            hydro_print "安装程序不支持 Kernel SU，请使用 Magisk"
            hydro_abort
            ;;
        en)
            hydro_print "Kernel SU not supported, please use Magisk instead!"
            hydro_abort
            ;;
        fr)
            hydro_print "Kernel SU n'est pas pris en charge, veuillez utiliser Magisk à la place !"
            hydro_abort
            ;;
        esac
    elif [[ "$APATCH" == "true" ]]; then
        hydro_print "APATCH: $APATCH_VER_CODE"
        case $lang in
        zh)
            hydro_print "未适配 APATCH，请使用 Magisk 以避免可能的风险！"
            hydro_abort
            ;;
        en)
            hydro_print "APATCH not supported, please use Magisk instead!"
            hydro_abort
            ;;
        fr)
            hydro_print "APATCH n'est pas pris en charge, veuillez utiliser Magisk à la place !"
            hydro_abort
            ;;
        esac
        sleep 5
    else
        hydro_print "Magisk: $MAGISK_VER_CODE"
        if [ "$MAGISK_VER_CODE" -lt 26000 ]; then
            case $lang in
            zh)
                hydro_print "请更新 Magisk 以使用本模块"
                hydro_abort
                ;;
            en)
                hydro_print "Outdated Magisk detected, please update!"
                hydro_abort
                ;;
            fr)
                hydro_print "Magisk obsolète détecté, veuillez mettre à jour !"
                hydro_abort
                ;;
            esac
        fi
    fi
    sleep 0.5
    print_line
    hydro_environment_detect
}

hydro_environment_detect(){
    sed -i '/# Test sed for Magisk/,/# Test sed for Magisk fin./d' "$MODPATH"/system.prop
    if [ "`sed -n '/# Test sed for Magisk/p' "$MODPATH"/system.prop`" ]; then
        print_line
        case "$lang" in
        zh)
            hydro_print "抱歉，你的 Magisk 不能正常工作，本模块将不会被安装！"
            ;;
        en)
            hydro_print "Sorry, your Magisk is not running properly, the module will not be installed! "
            ;;
        fr)
            hydro_print "Magisk ne fonctionne pas correctement et le module ne sera pas installé! "
            ;;
        esac
        print_line
        hydro_abort
    fi
}

hydro_unzip(){
    for file in "$MODPATH"/platform_config/$1/*; do
        cp -r $file "$MODPATH"
    done
}

hydro_model(){
    hydro_unzip "$model"
    #case "$model" in
    #21051182C) # 11
        
    #    ;;
    #23046RP50C)
        
    #    ;;
    #esac
}

unins_module(){
    rm -f "$1/service.sh"
    rm -f "$1/post-fs-data.sh"
    rm -f "$1/system.prop"
    touch "$1/remove"
}

unins_select(){
    case $lang in
    zh)
        [ "$5" = "-1" ] && hydro_print "检测到 $3，请卸载此人的模块，否则将终止安装"
        [ "$5" = "1" ] && hydro_print "检测到冲突，您愿意卸载来自 $3 的 $2 模块吗？"
        [ "$5" = "5" ] && hydro_print "检测到已启用冲突的功能，您愿意卸载来自 $3 的 $2 模块吗？"
        [ "$5" = "0" ] && hydro_print "检测到可能的冲突，您愿意卸载来自 $3 的 $2 模块吗？"
        [ "$5" = "0" ] && [ "$conflict_prompt" = "" ] && hydro_print "若您存在语言功能障碍，请自行查阅\"可能\"一词的含义"
        conflict_prompt="1"
        if [ $5 -ge 2 ]; then
            [ "$5" = "2" ] && hydro_print "检测到来自 $3 的 $2 模块存在与服务器通讯的行为"
            [ "$5" = "3" ] && hydro_print "检测到来自 $3 的 $2 模块存在使用 /dev/zero 的行为"
            [ "$5" = "4" ] && hydro_print "检测到来自 $3 的 $2 模块存在过长的 echo 命令，可能存在加密"
            hydro_print "请确保此模块的来源正当，否则存在损坏设备的风险"
            hydro_print "如果来源不明，请按音量键 + 以卸载"
        fi
        hydro_print "音量键 + : 是的"
        hydro_print "音量键 - : 不，我需要保留它"
        ;;
    en)
        [ "$5" = "-1" ] && hydro_print "$3 is detected, please uninstall his module, otherwise the installation will be forcibly cancelled"
        [ "$5" = "1" ] && hydro_print "Conflict detected, would you like to uninstall module $2 from $3?"
        [ "$5" = "5" ] && hydro_print "Conflicting enabled features detected, would you like to uninstall the $2 module from $3?"
        [ "$5" = "0" ] && hydro_print "Possible conflict detected, would you like to uninstall module $2 from $3?"
        if [ $5 -ge 2 ]; then
            [ "$5" = "2" ] && hydro_print "Detected module $2 from $3 communicating with server"
            [ "$5" = "3" ] && hydro_print "Detected behavior of module $2 from $3 using /dev/zero"
            [ "$5" = "4" ] && hydro_print "Detected excessively long echo command in $2 module from $3, possible encryption"
            hydro_print "Please ensure that this module comes from a trustworthy source,"
            hydro_print "otherwise there is a risk of damaging the device"
            hydro_print "If the source is unknown, press volume key + to uninstall"
        fi
        hydro_print "Volume + : Yes"
        hydro_print "Volume - : No, that's unacceptable"
        ;;
    fr)
        [ "$5" = "-1" ] && hydro_print "3$ sont détectés, veuillez désinstaller son module, sinon l'installation sera annulée de force"
        [ "$5" = "1" ] && hydro_print "Conflit détecté, souhaitez-vous désinstaller le module $2 de $3 ?"
        [ "$5" = "5" ] && hydro_print "Fonctionnalités activées en conflit détectées. Souhaitez-vous désinstaller le module à $2 de celui à $3 ?"
        [ "$5" = "0" ] && hydro_print "Conflit possible détecté, souhaitez-vous désinstaller le module $2 de $3 ?"
        if [ $5 -ge 2 ]; then
            [ "$5" = "2" ] && hydro_print "Module détecté 2 $ sur 3 $ communiquant avec le serveur"
            [ "$5" = "3" ] && hydro_print "Comportement détecté du module $2 à partir de $3 en utilisant /dev/zero"
            [ "$5" = "4" ] && hydro_print "Commande d'écho trop longue détectée dans le module $2 à partir de $3, cryptage possible"
            hydro_print "Veuillez vous assurer que ce module provient d'une source fiable,"
            hydro_print "sinon il y a un risque d'endommager l'appareil"
            hydro_print "Si la source est inconnue, appuyez sur la touche de volume + pour désinstaller"
        fi
        hydro_print "Préféreriez-vous désinstaller le module $2 apporté par $3 ?"
        hydro_print "Volume + : Oui"
        hydro_print "Volume - : Non, c'est inacceptable"
        ;;
    esac
    local key_click_unins
    while [ "$key_click_unins" = "" ]; do
        key_click_unins=$(getevent -qlc 1 | awk '{ print $3 }' | grep 'KEY_')
        sleep 0.2
    done
    case $key_click_unins in
    KEY_VOLUMEUP)
        unins_module $1
        case $lang in
        zh)
            hydro_print "卸载完成"
            ;;
        en)
            hydro_print "Uninstallation completed, well done!"
            ;;
        fr)
            hydro_print "Désinstallation terminée, bravo !"
            ;;
        esac
        ;;
    *)
        if [ "$4" = "1" ]; then
            case $lang in
            zh)
                hydro_print "安装终止"
                ;;
            en)
                hydro_print "Installation terminated since not compatible with the module"
                ;;
            fr)
                hydro_print "Installation terminée car non compatible avec le module"
                ;;
            esac
            hydro_abort
        else
            case $lang in
            zh)
                hydro_print "未卸载该模块"
                [ "$not_unins_prompt" = "" ] && hydro_print "请确保你的行为是安全的，或自行承担可能的风险"
                ;;
            en)
                hydro_print "Retained the module"
                [ "$not_unins_prompt" = "" ] && hydro_print "Please ensure that your behavior is safe or bear the possible risks at your own risk"
                ;;
            fr)
                hydro_print "Le module n'est pas désinstallé"
                [ "$not_unins_prompt" = "" ] && hydro_print "Veuillez vous assurer que votre comportement est sûr ou supporter les risques possibles à vos propres risques"
                ;;
            esac
            not_unins_prompt="1"
        fi
        ;;
    esac
    sleep 0.3
    print_line
}

unins_bypass(){
    local module_prop="$1/module.prop"
    local name=$(grep_prop name "$module_prop")
    local author=$(grep_prop author "$module_prop")
    case $lang in
    zh)
        hydro_print "检测到来自 $author 的 $name 模块存在放行标识"
        hydro_print "请确保此模块来源安全"
        ;;
    en)
        hydro_print "Bypass flag detected for module $name from $author"
        hydro_print "Please ensure that the source of this module is safe"
        ;;
    fr)
        hydro_print "Indicateur de contournement détecté pour le module $name à partir de $author"
        hydro_print "Veuillez vous assurer que la source de ce module est sûre"
        ;;
    esac
    print_line
}

uninstall_module(){
    local module_prop="$1/module.prop"
    local name=$(grep_prop name "$module_prop")
    local author=$(grep_prop author "$module_prop")
    unins_select "$1" "$name" "$author" "$2" "$3" "$4"
}

hydro_fakemodel_abort(){
    case $lang in
    zh)
        hydro_print "检测到机型伪装模块，请将其卸载，重启后重新安装本模块"
        ;;
    en)
        hydro_print "Device model camouflage module detected"
        hydro_print "Please uninstall it and reinstall the module after rebooting"
        ;;
    fr)
        hydro_print "Module de camouflage du modèle d'appareil détecté"
        hydro_print "Veuillez le désinstaller et réinstaller le module après le redémarrage"
        ;;
    esac
    hydro_abort
}

hydro_cure(){
    case $lang in
    zh)
        hydro_print "正在进行环境安全性检测，请耐心等待……"
        ;;
    en)
        hydro_print "Environmental safety testing is in progress, please wait patiently..."
        ;;
    fr)
        hydro_print "Les tests de sécurité environnementale sont en cours, veuillez patienter..."
        ;;
    esac
    print_line
    local dir
    local subdir
    for dir in /data/adb/modules /data/adb/modules_update; do
        for subdir in $(find "$dir" -maxdepth 1 -type d); do
            local module_prop="$subdir/module.prop"
            local system_prop="$subdir/system.prop"
            local service="$subdir/service.sh"
            local bypass="$subdir/ab_certified"
            local author=$(grep_prop author "$module_prop")
            local folder=$(basename "$subdir")
            if [ -f "$module_prop" -a ! -e "$bypass" -a ! -e "$subdir"/remove -a "$folder" != "ab_optimizer" -a "$folder" != "extreme_gt" -a "$author" != "嘟嘟ski" -a "$author" != "nakixii" ]; then
                if grep -q -i -e 'int萌新很新' -e '水竹闲士' -e 'SutoLiu' "$module_prop"; then
                    uninstall_module "$subdir" "1" "-1"
                elif grep -q -e 'ro.product.name' -e 'ro.product.brand' "$system_prop"; then
                    uninstall_module "$subdir" "1" "1"
                    hydro_fakemodel_abort
                elif find "$subdir"/ | grep -q -i -e 'dax-default.xml' -e 'audio-effects.xml' -e 'liuqin.xml'; then
                    uninstall_module "$subdir" "1" "1"
                fi
                if grep -q -i -e '屏幕' -e '触控' -e '超频响应' -e 'MEMC' -e 'ltpo' "$module_prop"; then
                    uninstall_module $subdir "0" "0"
                fi
                if grep -q 'curl' "$service"; then
                    uninstall_module $subdir "0" "2"
                elif grep -q '/dev/zero' "$service"; then
                    uninstall_module $subdir "0" "3"
                fi
                while IFS= read -r line; do
                    if echo "$line" | grep -q "^echo" && [ "${#line}" -gt 100 ]; then
                        uninstall_module $subdir "0" "4"
                        break
                    fi
                done < "$service"
            elif [ -e "$bypass" -o "$author" = "嘟嘟ski" -o "$author" = "nakixii" ]; then
                unins_bypass $subdir
            fi
        done
    done
    case $lang in
    zh)
        hydro_print "环境检测完毕"
        ;;
    en)
        hydro_print "Environmental testing completed"
        ;;
    fr)
        hydro_print "Tests environnementaux terminés"
        ;;
    esac
    print_line
}

hydro_disable_sfopt(){
    sed -i '/# Renderer optimize/,/# Renderer optimize fin./d' "$MODPATH"/system.prop
}

hydro_sfopt_install(){
    case $lang in
    zh)
        hydro_print "是否启用渲染优化？"
        hydro_print "本修改包括 surfaceflinger 等调整"
        hydro_print "提供更高效的图形处理、更低的操作时延和更平滑的动画体验"
        hydro_print "在过于旧的驱动和软件环境下可能会产生错误"
        hydro_print "音量键 + : 开启渲染优化"
        hydro_print "音量键 - : 关闭渲染优化"
        ;;
    en)
        hydro_print "Choose whether to enable renderer optimization"
        hydro_print "Including urfaceflinger optimization"
        hydro_print "Provides more efficient graphics processing,"
        hydro_print "lower operation latency and smoother animation experience"
        hydro_print "Errors may occur in older driver and software environments"
        hydro_print "Volume + : Enable renderer optimization"
        hydro_print "Volume - : Disable renderer optimization"
        ;;
    fr)
        hydro_print "Choisissez si vous souhaitez activer l'optimisation du moteur de rendu"
        hydro_print "Y compris l'optimisation d'urfaceflinger"
        hydro_print "Fournit un traitement graphique plus efficace,"
        hydro_print "une latence de fonctionnement plus faible et une expérience d'animation plus fluide"
        hydro_print "Des erreurs peuvent survenir dans les anciens environnements de pilotes et de logiciels"
        hydro_print "Volume + : Activer l'optimisation du moteur de rendu"
        hydro_print "Volume - : Désactiver l'optimisation du moteur de rendu"
        ;;
    esac
    while [ "$key_click_0" = "" ]; do
        key_click_0=$(getevent -qlc 1 | awk '{ print $3 }' | grep 'KEY_')
        sleep 0.2
    done
    case $key_click_0 in
    KEY_VOLUMEUP)
        hydro_print "-- ✔"
        ;;
    *)
        hydro_print "-- ✖"
        hydro_disable_sfopt
        ;;
    esac
    sleep 0.3
    print_line
}

hydro_enable_soundfx(){
    if [ "$model" = "liuqin" ]; then
        hydro_enable_soundfx_a
    else
        hydro_enable_soundfx_b
    fi
}

hydro_enable_soundfx_a(){
    cp -r -u "$MODPATH"/soundfx/a/system "$MODPATH"
    sed -i '/# Start of B-tune/,/# End of B-tune/d' "$MODPATH"/system.prop
}

hydro_enable_soundfx_b(){
    cp -r -u "$MODPATH"/soundfx/b/system "$MODPATH"
    sed -i '/# Start of A-tune/,/# End of A-tune/d' "$MODPATH"/system.prop
}

hydro_disable_soundfx(){
    rm -rf "$MODPATH"/soundfx
    sed -i '/## Ray/,/## End of tune/d' "$MODPATH"/system.prop
}

hydro_soundfx_install(){
    case $lang in
    zh)
        hydro_print "是否启用音频优化？"
        hydro_print "可绕过SRC，提供更低的延时和更高保真的音频表现"
        hydro_print "可能会导致一些未知问题，请自行尝试"
        hydro_print "音量键 + : 启用音频优化"
        hydro_print "音量键 - : 不启用音频优化"
        ;;
    en)
        hydro_print "Choose whether to enable sound effects modification"
        hydro_print "May perform abnormally on certain scenes"
        hydro_print "Volume + : Enable sound optimization"
        hydro_print "Volume - : Disable sound optimization"
        ;;
    fr)
        hydro_print "Choisissez si vous souhaitez activer la modification des effets sonores"
        hydro_print "Peut fonctionner anormalement sur certaines scènes"
        hydro_print "Volume + : Activer l'optimisation du son"
        hydro_print "Volume + : Désactiver l'optimisation du son"
        ;;
    esac
    while [ "$key_click_1" = "" ]; do
        key_click_1=$(getevent -qlc 1 | awk '{ print $3 }' | grep 'KEY_')
        sleep 0.2
    done
    case $key_click_1 in
    KEY_VOLUMEUP)
        hydro_print "-- ✔"
        case $lang in
        zh)
            hydro_print "--本功能主要由 Ray 完成开发"
            ;;
        en)
            hydro_print "--This feature is mainly developed by Ray"
            ;;
        fr)
            hydro_print "--Cette fonctionnalité est principalement développée par Ray"
            ;;
        esac
        hydro_enable_soundfx
        ;;
    *)
        hydro_print "-- ✖"
        hydro_disable_soundfx
        ;;
    esac
    sleep 0.5
    print_line
}

hydro_install_completed(){
    case $lang in
    zh)
        hydro_print "安装完成"
        hydro_print "请确保你已经阅读了以上全部的安装提示"
        hydro_print "加入QQ群聊 497806021 以反馈、交流和获取测试版更新"
        sleep 0.2
        hydro_print "现在，尽情欢呼吧！"
        hydro_print ""
        hydro_print ""
        hydro_print ""
        ;;
    en)
        hydro_print "Installation completed"
        hydro_print "Please ensure that you have been aware of the above precautions"
        hydro_print "Feedback and bug report via 981916119@qq.com"
        sleep 0.2
        hydro_print "Now, you will rejoice this! "
        hydro_print ""
        hydro_print ""
        hydro_print ""
        ;;
    fr)
        hydro_print "installation terminée"
        hydro_print "Veuillez vous assurer que vous avez pris connaissance des précautions ci-dessus"
        hydro_print "Commentaires et rapport de bug via 981916119@qq.com"
        sleep 0.2
        hydro_print "Maintenant, vous vous en réjouirez !"
        hydro_print ""
        hydro_print ""
        hydro_print ""
        ;;
    esac
}

hydro_settings_backup(){
    touch "$MODPATH"/settings.txt
    {
       echo "model:$model" 
       echo "0:$key_click_0"
       echo "1:$key_click_1"
       echo "en:$key_click_en"
       echo "fr:$key_click_fr"
       echo "}"
    } >> $settings
}

hydro_read_settings(){
    while read -r lineP; do
        case "$lineP" in
        device:*)
           param=${lineP#device:}
           device=$param
           ;;
        model:*)
           param=${lineP#model:}
           model=$param
           ;;
        0:*)
           param=${lineP#0:}
           key_click_0=$param
           ;;
        1:*)
           param=${lineP#1:}
           key_click_1=$param
           ;;
        en:*)
           param=${lineP#en:}
           key_click_en=$param
           ;;
        fr:*)
           param=${lineP#fr:}
           key_click_fr=$param
           ;;
        "}")
           break
           ;;
        esac
    done < $settings_old
}

hydro_apply_settings(){
    case $key_click_0 in
    KEY_VOLUMEUP)
        hydro_print "--Renderer optimization enabled"
        ;;
    *)
        hydro_print "--Renderer optimization disabled"
        hydro_disable_sfopt
        ;;
    esac
    case $key_click_1 in
    KEY_VOLUMEUP)
        hydro_print "--Sound optimization activated"
        hydro_enable_soundfx
        ;;
    *)
        hydro_print "--Sound optimization deactivated"
        hydro_disable_soundfx
        ;;
    esac
    case $key_click_en in
    KEY_VOLUMEUP)
        sed -i "s/^description=.*$/description=$DESC_ZH/" "$HYPROP"
        ;;
    *)
        case $key_click_fr in
        KEY_VOLUMEUP)
            sed -i "s/^description=.*$/description=$DESC_FR/" "$HYPROP"
            ;;
        *)
            sed -i "s/^description=.*$/description=$DESC_EN/" "$HYPROP"
            ;;
        esac
    esac
    hydro_post_install
}

hydro_pre_install(){
    return 0
}

hydro_post_install(){
    rm -rf "$MODPATH"/soundfx
    rm -rf "$MODPATH"/platform_config
    set_perm_recursive "$MODPATH"/system/vendor 0 0 0755 0644 u:object_r:vendor_configs_file:s0
}

hydro_start_install(){
    key_click_0=""
    key_click_1=""
    key_click_en=""
    key_click_fr=""
    hydro_language_select
    hydro_cure
    hydro_model_detect
    hydro_kernel_version
    hydro_model
    hydro_sfopt_install #0
    if [ -z "$not_ins_soundfx" ]; then
        hydro_soundfx_install #1
    else
        key_click_1="KEY_VOLUMEDOWN"
        hydro_disable_soundfx
    fi
    hydro_install_completed
    hydro_settings_backup
    hydro_post_install
}



if [ -f "/data/adb/modules/ab_optimizer/settings.txt" ]; then
    hydro_read_settings
    old_settings_apply_go="1"
elif [ -f "/data/adb/modules_update/ab_optimizer/settings.txt" ]; then
    settings_old=/data/adb/modules_update/ab_optimizer/settings.txt
    hydro_read_settings
    old_settings_apply_go="1"
else
    old_settings_apply_go="0"
fi

if [ "$device" != "" ]; then
    old_settings_apply_go="0"
fi

if [ "$old_settings_apply_go" = "1" ]; then
    case $key_click_en in
    KEY_VOLUMEUP)
        lang="zh"
        print_line
        hydro_print "检测到本模块的旧有配置"
        hydro_print "是否一键调用？"
        hydro_print "音量键 + : 是"
        hydro_print "音量键 - : 否"
        ;;
    KEY_VOLUMEDOWN)
        case $key_click_fr in
        KEY_VOLUMEUP)
            lang="fr"
            print_line
            hydro_print "Une configuration existante a été détectée"
            hydro_print "Voulez-vous l'appliquer en un clic?"
            hydro_print "Volume + : Oui"
            hydro_print "Volume - : Non"
            sed -i "s/Hydrostellaire 沨莹/Hydrostellaire/g" "$HYPROP"
            ;;
        *)
            lang="en"
            print_line
            hydro_print "An existing configuration has been detected"
            hydro_print "Do you want to apply it with one click?"
            hydro_print "Volume + : Yes"
            hydro_print "Volume - : No"
            sed -i "s/Hydrostellaire 沨莹/Hydrostellaire/g" "$HYPROP"
            ;;
        esac
        ;;
    esac
    if [ "$hydro_old_flag" != "1" ]; then
        while [ "$key_click_a" = "" ]; do
            key_click_a=$(getevent -qlc 1 | awk '{ print $3 }' | grep 'KEY_')
            sleep 0.2
        done
        sleep 0.3
        print_line
        case $key_click_a in
        KEY_VOLUMEUP)
            hydro_pre_install
            hydro_model_detect
            hydro_kernel_version
            hydro_model
            hydro_cure
            hydro_apply_settings
            cp $settings_old /data/adb/modules_update/ab_optimizer
            ;;
        *)
            hydro_print "--Reinstall"
            sleep 0.3
            reinstall_flag=1
            hydro_pre_install
            hydro_start_install
            ;;
        esac
    fi
else
    hydro_start_install
fi